
// Beachte: zur Ausführung des Programms, werden die folgenden Bibliotheken benötigt.
#include <Adafruit_MotorShield.h> // Bibliothek für Motorshild
#include <PS2X_lib.h> //Bibliothek für Kontroller

PS2X ps2x; // ertellt Element der PS2X Controller Klasse
int error = 0; 

Adafruit_MotorShield AFMSbot(0x60); // Jumper geschlossen
Adafruit_MotorShield AFMStop(0x61); // Kein Jumper vorhanden

// Unteres Motorshield: Schrittmotor2 an M3/M4 mit 200 Schritten
Adafruit_StepperMotor *Schrittmotor2 = AFMStop.getStepper(200, 2);

// Oberes Motorshield: Schrittmotor1 an M1/M2 mit 200 Schritten; Schrittmotor3 an M3/M4 mit 200 Schritten
Adafruit_StepperMotor *Schrittmotor3 = AFMSbot.getStepper(200, 2);
Adafruit_StepperMotor *Schrittmotor1 = AFMStop.getStepper(200, 1);


// Taster 1,2,3 und 4; bei Betätigung ist die Fahrt in die betroffene Richtung nicht mehr möglich
void setup() 
{
  pinMode(2, INPUT_PULLUP); // Durch INPUT_PULLUP wird das Verhalten des Input-Modus effektiv ugekehrt
  pinMode(3, INPUT_PULLUP); // => HIGH - Sensor nicht betätigt, LOW - Sensor betätigt
  pinMode(4, INPUT_PULLUP);
  pinMode(5, INPUT_PULLUP);

  AFMSbot.begin(); // Initialisiert unteres Motorshield
  AFMStop.begin(); // Initialisiert oberes Motorshield
   
  while (!Serial);
  
  Serial.begin(57600);

  error = ps2x.config_gamepad(13,11,10,12, true, true);   // Richtet Pins und Einstellungen ein und prüft Gamepad auf Fehler
}
int i;
void loop() 
{
  // error Handling
  if(error == 1) //Schleife überspringen, wenn kein Controller gefunden wurde 
    return; 
  else // wenn Controller gefunden wurde:
  { 
    ps2x.read_gamepad(false, 0); //Controller auslesen
  }

  // Bei der Ansteuerung von Motoren über ein Motorshields gibt es verschiedene Möglichkeiten.
  // Für die Verwendete Form, werden die "Drehrichtung" und der "Schritttyp" benötigt.
  // Die Drehrichtung kann durch "FORWARD" (vorwärts) / "BACKWARD" (rückwärts) bestimmt werden.
  // Der Schritttyp wird durch "INTERLEAVE/DOUBLE/SINGLE/MICROSTEP" bestimmt.
  // wobei gilt:
  //        
  //  SINGLE     - "Single-Coil-Aktivierung" -> wenig Stromverbrauch
  //  DOUBLE     - "Double-Coil-Aktivierung" -> höheres Drehmoment
  //  INTERLEAVE - Mix aus SINGLE und DOUBLE -> Wechsel zwischen SINGLE und DOUBLE -> halbe Geschwindigkeit! -> genauer
  //  MICROSTEP  - Micro Steps - PWM-Steuerung -> flüssige Bewegung zwischen den Schritten -> sehr genau

 
  //############################## MOTOR 1 - (Ausführung der horizontalen Fahrt) ############
  if(!digitalRead(2)) // wird ausgeführt solange der Taster nicht betätigt wurde
  {
    if(ps2x.Analog(PSS_RX) > 130)
      Schrittmotor1->onestep(FORWARD, MICROSTEP);    // bei schwacher Auslenkung des Joysticks -> langsam
    if(ps2x.Analog(PSS_RX) > 200)
      Schrittmotor1->onestep(FORWARD, INTERLEAVE);  // bei starker Auslenkung des Joysticks -> schnell
  }
  
  if(!digitalRead(3)) // wird ausgeführt solange der Taster nicht betätigt wurde
  {
    if(ps2x.Analog(PSS_RX)<50)
      Schrittmotor1->onestep(BACKWARD, INTERLEAVE); 
    if(ps2x.Analog(PSS_RX)<120)
      Schrittmotor1->onestep(BACKWARD, MICROSTEP); 
  }
  
  if(ps2x.Analog(PSS_RX)==128)
    Schrittmotor1->release(); // Haltestrom der Motoren abschalten, wenn der rechte Joystick nicht ausgelenkt wird -> Schlitten kann manuell verschoben werden

//################ MOTOR 2 (Ausführung der Drehung um die Y-Achse) ########################## 
// Steuerung über linken Joystick (X-Achse)  <128 = Auslenkung nach "links"; 128 = keine Auslenkung; >128 = Auslenkung nach "rechts"  
  int value = ps2x.Analog(PSS_LX); 
  
    if(value == 255)
      Schrittmotor2->onestep(FORWARD, DOUBLE);      // bei starker Auslenkung des Joysticks -> schnell
    else if(value > 128)
      Schrittmotor2->onestep(FORWARD, MICROSTEP);   // bei schwacher Auslenkung des Joysticks -> langsam
  
    if(value == 0)
      Schrittmotor2->onestep(BACKWARD, INTERLEAVE); 
    else if(value < 128)
      Schrittmotor2->onestep(BACKWARD, MICROSTEP); 
      
//############### MOTOR 3 (Ausführung der Drehung um die X-Achse) ########################### 
// Steuerung über linken Joystick (Y-Achse)  <128 = Auslenkung nach "oben"; 128 = keine Auslenkung; >128 = Auslenkung nach "unten" 
 value = ps2x.Analog(PSS_LY); 
  
  if(!digitalRead(5))  /// wird ausgeführt solange der Taster nicht betätigt wurde
  {
    if(value == 255)
      Schrittmotor3->onestep(FORWARD, INTERLEAVE);  // bei starker Auslenkung des Joysticks -> schnell
    else if(value > 128)
      Schrittmotor3->onestep(FORWARD, MICROSTEP);   // bei schwacher Auslenkung des Joysticks -> langsam
  }
  
  if(!digitalRead(4))  // wird ausgeführt solange der Taster nicht betätigt wurde
  {
    if(value == 0)
      Schrittmotor3->onestep(BACKWARD, INTERLEAVE); 
    else if(value < 127)
      Schrittmotor3->onestep(BACKWARD, MICROSTEP); 
  }   
//###########################################################################################
  
delay(10);    
}
